@@ -1,8 +1,10 @@
 from compliance_checker.acdd import ACDD1_1Check, ACDD1_3Check
+from compliance_checker.tests.helpers import MockTimeSeries, MockVariable
 from compliance_checker.tests.resources import STATIC_FILES
 from compliance_checker.tests import BaseTestCase
 from netCDF4 import Dataset
 import os
+import numpy as np
 
 
 def to_singleton_var(l):
@@ -13,7 +15,6 @@ def to_singleton_var(l):
     return [x[0] if hasattr(x, '__iter__') and not isinstance(x, str) else x
             for x in l]
 
-
 def check_varset_nonintersect(group0, group1):
     '''
     Returns true if both groups contain the same elements, regardless of
@@ -27,7 +28,7 @@ def check_varset_nonintersect(group0, group1):
 
 
 class TestACDD1_1(BaseTestCase):
-
+    # TODO superclass this so ACDD1_3 can inherit?
     # Adapted using `pandas.read_html` from URL
     # http://wiki.esipfed.org/index.php/Attribute_Convention_for_Data_Discovery_1-1
     expected = {
@@ -108,8 +109,12 @@ class TestACDD1_1(BaseTestCase):
                                          self.acdd_highly_recommended)
 
         # Check the reference dataset, NCEI 1.1 Gold Standard Point
+        missing = ['"Conventions" does not contain \'ACDD-1.3\'']
         results = self.acdd.check_high(self.ds)
         for result in results:
+            if result.msgs and all([m in missing for m in result.msgs]):
+                # only the Conventions check should have failed
+                 self.assert_result_is_bad(result)
             self.assert_result_is_good(result)
 
         # Create an empty dataset that writes to /dev/null This acts as a
@@ -130,20 +135,18 @@ class TestACDD1_1(BaseTestCase):
                                          self.acdd_recommended)
 
         ncei_exceptions = [
-            'geospatial_bounds',
-            'time_coverage_duration'
+            'geospatial_bounds not present',
+            'time_coverage_duration not present',
+            'time_coverage_resolution not present'
         ]
         results = self.acdd.check_recommended(self.ds)
         for result in results:
-            # NODC 1.1 doesn't have some ACDD attributes
-            if result.name in ncei_exceptions:
-                continue
-
-            # The NCEI Gold Standard Point is missing time_coverage_resolution...
-            if result.name == 'time_coverage_resolution':
+            if (result.msgs) and all([m in ncei_exceptions for m in result.msgs]): # we're doing string comparisons, this is kind of hacky...
                 self.assert_result_is_bad(result)
                 continue
 
+            self.assert_result_is_good(result)
+
         # Create an empty dataset that writes to /dev/null This acts as a
         # temporary netCDF file in-memory that never gets written to disk.
         empty_ds = Dataset(os.devnull, 'w', diskless=True)
@@ -162,14 +165,14 @@ class TestACDD1_1(BaseTestCase):
 
         # Attributes that are missing from NCEI but should be there
         missing = [
-            'geospatial_lat_resolution',
-            'geospatial_lon_resolution',
-            'geospatial_vertical_resolution'
+            'geospatial_lat_resolution not present',
+            'geospatial_lon_resolution not present',
+            'geospatial_vertical_resolution not present'
         ]
 
         results = self.acdd.check_suggested(self.ds)
         for result in results:
-            if result.name in missing:
+            if (result.msgs) and all([m in missing for m in result.msgs]): # we're doing string comparisons, this is kind of hacky...
                 self.assert_result_is_bad(result)
                 continue
 
@@ -185,6 +188,7 @@ class TestACDD1_1(BaseTestCase):
             self.assert_result_is_bad(result)
 
     def test_acknowldegement_check(self):
+        """Check both the British- and American-English spellings of 'acknowledgement'"""
         # Check British Spelling
         try:
             empty0 = Dataset(os.devnull, 'w', diskless=True)
@@ -198,7 +202,7 @@ class TestACDD1_1(BaseTestCase):
             empty0.close()
 
         try:
-            # Check British spelling
+            # Check American spelling
             empty1 = Dataset(os.devnull, 'w', diskless=True)
             result = self.acdd.check_acknowledgment(empty1)
             self.assert_result_is_bad(result)
@@ -287,6 +291,7 @@ class TestACDD1_3(BaseTestCase):
         ]
     }
 
+
     def setUp(self):
         # Use the NCEI Gold Standard Point dataset for ACDD checks
         self.ds = self.load_dataset(STATIC_FILES['ncei_gold_point_2'])
@@ -321,11 +326,11 @@ class TestACDD1_3(BaseTestCase):
 
         results = self.acdd.check_recommended(self.ds)
         ncei_exceptions = [
-            'time_coverage_duration',
-            'time_coverage_resolution'
+            'time_coverage_duration not present',
+            'time_coverage_resolution not present'
         ]
         for result in results:
-            if result.name in ncei_exceptions:
+            if (result.msgs) and all([m in ncei_exceptions for m in result.msgs]): # we're doing string comparisons, this is kind of hacky...
                 self.assert_result_is_bad(result)
                 continue
 
@@ -341,12 +346,12 @@ class TestACDD1_3(BaseTestCase):
         results = self.acdd.check_suggested(self.ds)
         # NCEI does not require or suggest resolution attributes
         ncei_exceptions = [
-            'geospatial_lat_resolution',
-            'geospatial_lon_resolution',
-            'geospatial_vertical_resolution'
+            'geospatial_lat_resolution not present',
+            'geospatial_lon_resolution not present',
+            'geospatial_vertical_resolution not present'
         ]
         for result in results:
-            if result.name in ncei_exceptions:
+            if (result.msgs) and all([m in ncei_exceptions for m in result.msgs]): # we're doing string comparisons, this is kind of hacky...
                 self.assert_result_is_bad(result)
                 continue
             self.assert_result_is_good(result)
@@ -395,20 +400,6 @@ class TestACDD1_3(BaseTestCase):
         for result in results:
             self.assert_result_is_bad(result)
 
-    def test_acknowledgement(self):
-        '''
-        Test acknowledgement attribute is being checked
-        '''
-
-        empty_ds = Dataset(os.devnull, 'w', diskless=True)
-        self.addCleanup(empty_ds.close)
-
-        result = self.acdd.check_acknowledgment(self.ds)
-        self.assert_result_is_good(result)
-
-        result = self.acdd.check_acknowledgment(empty_ds)
-        self.assert_result_is_bad(result)
-
     def test_vertical_extents(self):
         '''
         Test vertical extents are being checked
@@ -416,3 +407,167 @@ class TestACDD1_3(BaseTestCase):
 
         result = self.acdd.check_vertical_extents(self.ds)
         self.assert_result_is_good(result)
+
+    def test_geospatial_bounds(self):
+        '''
+        Test geospatial bounds are checked and provide a good error message
+        '''
+        # Create an empty dataset that writes to /dev/null This acts as a
+        # temporary netCDF file in-memory that never gets written to disk.
+        empty_ds = Dataset(os.devnull, 'w', diskless=True)
+        self.addCleanup(empty_ds.close)
+
+        # Mispelled WKT. Error message should include the attribute checked
+        # and the value that was provided for easy troubleshooting
+        empty_ds.geospatial_bounds = "POIT (-123.458000 38.048000)"
+        results = self.acdd.check_recommended(empty_ds)
+        for result in results:
+            if result.variable_name == 'geospatial_bounds':
+                assert ('Could not parse WKT from geospatial_bounds,'
+                        ' possible bad value: "{}"'.format(empty_ds.geospatial_bounds) in result.msgs)
+
+    def test_time_extents(self):
+        '''
+        Test that the time extents are being checked
+        '''
+        result = self.acdd.check_time_extents(self.ds)
+        self.assert_result_is_good(result)
+
+        empty_ds = Dataset(os.devnull, 'w', diskless=True)
+        self.addCleanup(empty_ds.close)
+
+        # The dataset needs at least one variable to check that it's missing
+        # all the required attributes.
+        empty_ds.createDimension('time', 1)
+        time_var = empty_ds.createVariable('time', 'float32', ('time',))
+        time_var.units = 'seconds since 1970-01-01 00:00:00 UTC'
+        time_var[:] = [1451692800]  # 20160102T000000Z in seconds since epoch
+        empty_ds.time_coverage_start = '20160102T000000Z'
+        empty_ds.time_coverage_end = '20160102T000000Z'
+        result = self.acdd.check_time_extents(empty_ds)
+        self.assert_result_is_good(result)
+        # try the same thing with time offsets
+        time_var.units = 'seconds since 1970-01-01 00:00:00-10:00'
+        empty_ds.time_coverage_start = '20160102T000000-1000'
+        empty_ds.time_coverage_end = '20160102T000000-1000'
+        result = self.acdd.check_time_extents(empty_ds)
+        self.assert_result_is_good(result)
+
+    def test_check_lat_extents(self):
+        """Test the check_lat_extents() method behaves expectedly"""
+
+        # create dataset using MockDataset, give it lat/lon dimensions
+        ds = MockTimeSeries()
+        ds.variables["lat"][:] = np.linspace(-135., -130., num=500) # arbitrary, but matches time dim size
+
+        # test no values, expect failure
+        result = self.acdd.check_lat_extents(ds)
+        self.assert_result_is_bad(result)
+
+        # give integer geospatial_lat_max/min, test
+        ds.setncattr("geospatial_lat_min", -135)
+        ds.setncattr("geospatial_lat_max", -130)
+
+        result = self.acdd.check_lat_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give float geospatial_lat_min/max, test
+        ds.setncattr("geospatial_lat_min", -135.)
+        ds.setncattr("geospatial_lat_max", -130.)
+
+        result = self.acdd.check_lat_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give string (in number-form), test
+        ds.setncattr("geospatial_lat_min", "-135.")
+        ds.setncattr("geospatial_lat_max", "-130.")
+
+        result = self.acdd.check_lat_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give garbage string -- expect failure
+        ds.setncattr("geospatial_lat_min", "bad")
+        ds.setncattr("geospatial_lat_max", "val")
+
+        result = self.acdd.check_lat_extents(ds)
+        self.assert_result_is_bad(result)
+
+    def test_check_lon_extents(self):
+        """Test the check_lon_extents() method behaves expectedly"""
+
+        # create dataset using MockDataset, give it lat/lon dimensions
+        ds = MockTimeSeries()
+        ds.variables["lon"][:] = np.linspace(65., 67., num=500)
+
+        # test no values, expect failure
+        #result = self.acdd.check_lon_extents(ds)
+        #self.assert_result_is_bad(result)
+
+        # give integer geospatial_lon_max/min, test
+        ds.setncattr("geospatial_lon_min", 65)
+        ds.setncattr("geospatial_lon_max", 67)
+
+        result = self.acdd.check_lon_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give float geospatial_lon_min/max, test
+        ds.setncattr("geospatial_lon_min", 65.)
+        ds.setncattr("geospatial_lon_max", 67.)
+
+        result = self.acdd.check_lon_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give string (in number-form), test
+        ds.setncattr("geospatial_lon_min", "65.")
+        ds.setncattr("geospatial_lon_max", "67.")
+
+        result = self.acdd.check_lon_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give garbage string -- expect failure
+        ds.setncattr("geospatial_lon_min", "bad")
+        ds.setncattr("geospatial_lon_max", "val")
+
+        result = self.acdd.check_lon_extents(ds)
+        self.assert_result_is_bad(result)
+
+    def test_check_geospatial_vertical_max(self):
+        ds = MockTimeSeries()
+        ds.variables["depth"][:] = np.linspace(0., 30., num=500)
+
+        # give integer geospatial_vertical_max/min, test
+        ds.setncattr("geospatial_vertical_min", 0)
+        ds.setncattr("geospatial_vertical_max", 30)
+
+        result = self.acdd.check_vertical_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give float geospatial_vertical_min/max, test
+        ds.setncattr("geospatial_vertical_min", 0.)
+        ds.setncattr("geospatial_vertical_max", 30.)
+
+        result = self.acdd.check_vertical_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give string (in number-form), test
+        ds.setncattr("geospatial_vertical_min", "0.")
+        ds.setncattr("geospatial_vertical_max", "30.")
+
+        result = self.acdd.check_vertical_extents(ds)
+        self.assert_result_is_good(result)
+
+        # give garbage string -- expect failure
+        ds.setncattr("geospatial_vertical_min", "bad")
+        ds.setncattr("geospatial_vertical_max", "val")
+
+        result = self.acdd.check_vertical_extents(ds)
+        self.assert_result_is_bad(result)
+
+        # all masked values mean that there are no valid array elements to get
+        # the min/max of
+        ds.setncattr("geospatial_vertical_min", 0.)
+        ds.setncattr("geospatial_vertical_max", 30.)
+        ds.variables["depth"][:] = np.ma.masked_all(ds.variables["depth"]
+                                                    .shape)
+        result = self.acdd.check_vertical_extents(ds)
+        self.assert_result_is_bad(result)
